@@ -1,11 +1,18 @@
+from __future__ import annotations
+
 from copy import deepcopy
+from typing import Any, Iterable, Sequence
 
 from django import forms
+from django.db.models import Field
 from django.contrib import admin
 from django.contrib.admin.options import BaseModelAdmin, InlineModelAdmin, flatten_fieldsets
 from django.contrib.contenttypes.admin import GenericStackedInline, GenericTabularInline
+from django.forms.models import BaseInlineFormSet
+from django.http.request import HttpRequest
 
 from modeltranslation import settings as mt_settings
+from modeltranslation.fields import TranslationField
 from modeltranslation.translator import translator
 from modeltranslation.utils import (
     build_css_class,
@@ -16,44 +23,51 @@ from modeltranslation.utils import (
     unique,
 )
 from modeltranslation.widgets import ClearableWidgetWrapper
+from modeltranslation._typing import _ListOrTuple
 
 
 class TranslationBaseModelAdmin(BaseModelAdmin):
-    _orig_was_required = {}
+    _orig_was_required: dict[str, bool] = {}
     both_empty_values_fields = ()
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
         self.trans_opts = translator.get_options_for_model(self.model)
         self._patch_prepopulated_fields()
 
-    def _get_declared_fieldsets(self, request, obj=None):
+    def _get_declared_fieldsets(
+        self, request: HttpRequest, obj: Any | None = None
+    ) -> _ListOrTuple[tuple[str | None, dict[str, Any]]] | None:
         # Take custom modelform fields option into account
-        if not self.fields and hasattr(self.form, '_meta') and self.form._meta.fields:
+        if not self.fields and hasattr(self.form, "_meta") and self.form._meta.fields:
             self.fields = self.form._meta.fields
         # takes into account non-standard add_fieldsets attribute used by UserAdmin
         fieldsets = (
             self.add_fieldsets
-            if getattr(self, 'add_fieldsets', None) and obj is None
+            if getattr(self, "add_fieldsets", None) and obj is None
             else self.fieldsets
         )
         if fieldsets:
             return self._patch_fieldsets(fieldsets)
         elif self.fields:
-            return [(None, {'fields': self.replace_orig_field(self.get_fields(request, obj))})]
+            return [(None, {"fields": self.replace_orig_field(self.get_fields(request, obj))})]
         return None
 
-    def formfield_for_dbfield(self, db_field, request, **kwargs):
+    def formfield_for_dbfield(
+        self, db_field: Field, request: HttpRequest, **kwargs: Any
+    ) -> forms.Field:
         field = super().formfield_for_dbfield(db_field, request, **kwargs)
-        self.patch_translation_field(db_field, field, request, **kwargs)
-        return field
+        self.patch_translation_field(db_field, field, request, **kwargs)  # type: ignore[arg-type]
+        return field  # type: ignore[return-value]
 
-    def patch_translation_field(self, db_field, field, request, **kwargs):
+    def patch_translation_field(
+        self, db_field: Field, field: forms.Field, request: HttpRequest, **kwargs: Any
+    ) -> None:
         if db_field.name in self.trans_opts.fields:
             if field.required:
                 field.required = False
                 field.blank = True
-                self._orig_was_required['%s.%s' % (db_field.model._meta, db_field.name)] = True
+                self._orig_was_required["%s.%s" % (db_field.model._meta, db_field.name)] = True
 
         # For every localized field copy the widget from the original field
         # and add a css class to identify a modeltranslation widget.
@@ -67,7 +81,8 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
             attrs = field.widget.attrs
             # if any widget attrs are defined on the form they should be copied
             try:
-                field.widget = deepcopy(self.form._meta.widgets[orig_field.name])  # this is a class
+                # this is a class:
+                field.widget = deepcopy(self.form._meta.widgets[orig_field.name])  # type: ignore[index]
                 if isinstance(field.widget, type):  # if not initialized
                     field.widget = field.widget(attrs)  # initialize form widget with attrs
             except (AttributeError, TypeError, KeyError):
@@ -83,24 +98,24 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
                         b for b in form_class.__bases__ if b != NullCharField
                     )
                 field.__class__ = type(
-                    'Nullable%s' % form_class.__name__, (NullableField, form_class), {}
+                    "Nullable%s" % form_class.__name__, (NullableField, form_class), {}
                 )
             if (
-                db_field.empty_value == 'both' or orig_field.name in self.both_empty_values_fields
+                db_field.empty_value == "both" or orig_field.name in self.both_empty_values_fields
             ) and isinstance(field.widget, (forms.TextInput, forms.Textarea)):
                 field.widget = ClearableWidgetWrapper(field.widget)
-            css_classes = self._get_widget_from_field(field).attrs.get('class', '').split(' ')
-            css_classes.append('mt')
+            css_classes = self._get_widget_from_field(field).attrs.get("class", "").split(" ")
+            css_classes.append("mt")
             # Add localized fieldname css class
-            css_classes.append(build_css_class(db_field.name, 'mt-field'))
+            css_classes.append(build_css_class(db_field.name, "mt-field"))
             # Add mt-bidi css class if language is bidirectional
             if get_language_bidi(db_field.language):
-                css_classes.append('mt-bidi')
+                css_classes.append("mt-bidi")
             if db_field.language == mt_settings.DEFAULT_LANGUAGE:
                 # Add another css class to identify a default modeltranslation widget
-                css_classes.append('mt-default')
+                css_classes.append("mt-default")
                 if orig_formfield.required or self._orig_was_required.get(
-                    '%s.%s' % (orig_field.model._meta, orig_field.name)
+                    "%s.%s" % (orig_field.model._meta, orig_field.name)
                 ):
                     # In case the original form field was required, make the
                     # default translation field required instead.
@@ -111,16 +126,16 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
                     # Hide clearable widget for required fields
                     if isinstance(field.widget, ClearableWidgetWrapper):
                         field.widget = field.widget.widget
-            self._get_widget_from_field(field).attrs['class'] = ' '.join(css_classes)
+            self._get_widget_from_field(field).attrs["class"] = " ".join(css_classes)
 
-    def _get_widget_from_field(self, field):
+    def _get_widget_from_field(self, field: forms.Field) -> Any:
         # retrieve "nested" widget in case of related field
         if isinstance(field.widget, admin.widgets.RelatedFieldWidgetWrapper):
             return field.widget.widget
         else:
             return field.widget
 
-    def _exclude_original_fields(self, exclude=None):
+    def _exclude_original_fields(self, exclude: _ListOrTuple[str] | None = None) -> tuple[str, ...]:
         if exclude is None:
             exclude = tuple()
         if exclude:
@@ -128,7 +143,7 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
             return exclude_new + tuple(self.trans_opts.fields.keys())
         return tuple(self.trans_opts.fields.keys())
 
-    def replace_orig_field(self, option):
+    def replace_orig_field(self, option: Iterable[str | Sequence[str]]) -> _ListOrTuple[str]:
         """
         Replaces each original field in `option` that is registered for
         translation by its translation fields.
@@ -158,36 +173,38 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
             for opt in option:
                 if opt in self.trans_opts.fields:
                     index = option_new.index(opt)
-                    option_new[index : index + 1] = get_translation_fields(opt)
+                    option_new[index : index + 1] = get_translation_fields(opt)  # type: ignore[arg-type]
                 elif isinstance(opt, (tuple, list)) and (
                     [o for o in opt if o in self.trans_opts.fields]
                 ):
                     index = option_new.index(opt)
                     option_new[index : index + 1] = self.replace_orig_field(opt)
             option = option_new
-        return option
+        return option  # type: ignore[return-value]
 
-    def _patch_fieldsets(self, fieldsets):
+    def _patch_fieldsets(
+        self, fieldsets: _ListOrTuple[tuple[str | None, dict[str, Any]]]
+    ) -> _ListOrTuple[tuple[str | None, dict[str, Any]]]:
         if fieldsets:
             fieldsets_new = list(fieldsets)
             for name, dct in fieldsets:
-                if 'fields' in dct:
-                    dct['fields'] = self.replace_orig_field(dct['fields'])
+                if "fields" in dct:
+                    dct["fields"] = self.replace_orig_field(dct["fields"])
             fieldsets = fieldsets_new
         return fieldsets
 
-    def _patch_prepopulated_fields(self):
-        def localize(sources, lang):
+    def _patch_prepopulated_fields(self) -> None:
+        def localize(sources: Sequence[str], lang: str) -> tuple[str, ...]:
             "Append lang suffix (if applicable) to field list"
 
-            def append_lang(source):
+            def append_lang(source: str) -> str:
                 if source in self.trans_opts.fields:
                     return build_localized_fieldname(source, lang)
                 return source
 
             return tuple(map(append_lang, sources))
 
-        prepopulated_fields = {}
+        prepopulated_fields: dict[str, Sequence[str]] = {}
         for dest, sources in self.prepopulated_fields.items():
             if dest in self.trans_opts.fields:
                 for lang in mt_settings.AVAILABLE_LANGUAGES:
@@ -198,7 +215,9 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
                 prepopulated_fields[dest] = localize(sources, lang)
         self.prepopulated_fields = prepopulated_fields
 
-    def _get_form_or_formset(self, request, obj, **kwargs):
+    def _get_form_or_formset(
+        self, request: HttpRequest, obj: Any | None, **kwargs: Any
+    ) -> dict[str, Any]:
         """
         Generic code shared by get_form and get_formset.
         """
@@ -208,7 +227,7 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
         else:
             exclude = list(exclude)
         exclude.extend(self.get_readonly_fields(request, obj))
-        if not exclude and hasattr(self.form, '_meta') and self.form._meta.exclude:
+        if not exclude and hasattr(self.form, "_meta") and self.form._meta.exclude:
             # Take the custom ModelForm's Meta.exclude into account only if the
             # ModelAdmin doesn't define its own.
             exclude.extend(self.form._meta.exclude)
@@ -216,27 +235,33 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
         # default on modelform_factory
         exclude = self.replace_orig_field(exclude) or None
         exclude = self._exclude_original_fields(exclude)
-        kwargs.update({'exclude': exclude})
+        kwargs.update({"exclude": exclude})
 
         return kwargs
 
-    def _get_fieldsets_pre_form_or_formset(self, request, obj=None):
+    def _get_fieldsets_pre_form_or_formset(
+        self, request: HttpRequest, obj: Any | None = None
+    ) -> _ListOrTuple[tuple[str | None, dict[str, Any]]] | None:
         """
         Generic get_fieldsets code, shared by
         TranslationAdmin and TranslationInlineModelAdmin.
         """
         return self._get_declared_fieldsets(request, obj)
 
-    def _get_fieldsets_post_form_or_formset(self, request, form, obj=None):
+    def _get_fieldsets_post_form_or_formset(
+        self, request: HttpRequest, form: type[forms.ModelForm], obj: Any | None = None
+    ) -> list:
         """
         Generic get_fieldsets code, shared by
         TranslationAdmin and TranslationInlineModelAdmin.
         """
         base_fields = self.replace_orig_field(form.base_fields.keys())
         fields = list(base_fields) + list(self.get_readonly_fields(request, obj))
-        return [(None, {'fields': self.replace_orig_field(fields)})]
+        return [(None, {"fields": self.replace_orig_field(fields)})]
 
-    def get_translation_field_excludes(self, exclude_languages=None):
+    def get_translation_field_excludes(
+        self, exclude_languages: list[str] | None = None
+    ) -> tuple[TranslationField, ...]:
         """
         Returns a tuple of translation field names to exclude based on
         `exclude_languages` arg.
@@ -254,7 +279,9 @@ class TranslationBaseModelAdmin(BaseModelAdmin):
                     exclude.append(tfield)
         return tuple(exclude)
 
-    def get_readonly_fields(self, request, obj=None):
+    def get_readonly_fields(
+        self, request: HttpRequest, obj: Any | None = None
+    ) -> _ListOrTuple[str]:
         """
         Hook to specify custom readonly fields.
         """
@@ -265,11 +292,11 @@ class TranslationAdmin(TranslationBaseModelAdmin, admin.ModelAdmin):
     # TODO: Consider addition of a setting which allows to override the fallback to True
     group_fieldsets = False
 
-    def __init__(self, *args, **kwargs):
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
         self._patch_list_editable()
 
-    def _patch_list_editable(self):
+    def _patch_list_editable(self) -> None:
         if self.list_editable:
             editable_new = list(self.list_editable)
             display_new = list(self.list_display)
@@ -283,7 +310,7 @@ class TranslationAdmin(TranslationBaseModelAdmin, admin.ModelAdmin):
             self.list_editable = editable_new
             self.list_display = display_new
 
-    def _group_fieldsets(self, fieldsets):
+    def _group_fieldsets(self, fieldsets: list) -> list:
         # Fieldsets are not grouped by default. The function is activated by
         # setting TranslationAdmin.group_fieldsets to True. If the admin class
         # already defines a fieldset, we leave it alone and assume the author
@@ -292,17 +319,18 @@ class TranslationAdmin(TranslationBaseModelAdmin, admin.ModelAdmin):
             flattened_fieldsets = flatten_fieldsets(fieldsets)
 
             # Create a fieldset to group each translated field's localized fields
-            fields = sorted(f for f in self.opts.get_fields() if f.concrete)
+            fields = sorted(f for f in self.opts.get_fields() if f.concrete)  # type: ignore[type-var]
             untranslated_fields = [
                 f.name
                 for f in fields
-                if (
+                if
+                (
                     # Exclude the primary key field
                     f is not self.opts.auto_field
                     # Exclude non-editable fields
                     and f.editable
                     # Exclude the translation fields
-                    and not hasattr(f, 'translated_field')
+                    and not hasattr(f, "translated_field")
                     # Honour field arguments. We rely on the fact that the
                     # passed fieldsets argument is already fully filtered
                     # and takes options like exclude into account.
@@ -313,8 +341,8 @@ class TranslationAdmin(TranslationBaseModelAdmin, admin.ModelAdmin):
             fieldsets = (
                 [
                     (
-                        '',
-                        {'fields': untranslated_fields},
+                        "",
+                        {"fields": untranslated_fields},
                     )
                 ]
                 if untranslated_fields
@@ -331,13 +359,13 @@ class TranslationAdmin(TranslationBaseModelAdmin, admin.ModelAdmin):
                     label = self.model._meta.get_field(orig_field).verbose_name.capitalize()
                     temp_fieldsets[orig_field] = (
                         label,
-                        {'fields': trans_fieldnames, 'classes': ('mt-fieldset',)},
+                        {"fields": trans_fieldnames, "classes": ("mt-fieldset",)},
                     )
 
             fields_order = unique(
                 f.translated_field.name
                 for f in self.opts.fields
-                if hasattr(f, 'translated_field') and f.name in flattened_fieldsets
+                if hasattr(f, "translated_field") and f.name in flattened_fieldsets
             )
             for field_name in fields_order:
                 fieldsets.append(temp_fieldsets.pop(field_name))
@@ -345,11 +373,15 @@ class TranslationAdmin(TranslationBaseModelAdmin, admin.ModelAdmin):
 
         return fieldsets
 
-    def get_form(self, request, obj=None, **kwargs):
+    def get_form(
+        self, request: HttpRequest, obj: Any | None = None, **kwargs: Any
+    ) -> type[forms.ModelForm]:
         kwargs = self._get_form_or_formset(request, obj, **kwargs)
         return super().get_form(request, obj, **kwargs)
 
-    def get_fieldsets(self, request, obj=None):
+    def get_fieldsets(
+        self, request: HttpRequest, obj: Any | None = None
+    ) -> _ListOrTuple[tuple[str | None, dict[str, Any]]]:
         return self._get_fieldsets_pre_form_or_formset(request, obj) or self._group_fieldsets(
             self._get_fieldsets_post_form_or_formset(
                 request, self.get_form(request, obj, fields=None), obj
@@ -358,11 +390,13 @@ class TranslationAdmin(TranslationBaseModelAdmin, admin.ModelAdmin):
 
 
 class TranslationInlineModelAdmin(TranslationBaseModelAdmin, InlineModelAdmin):
-    def get_formset(self, request, obj=None, **kwargs):
+    def get_formset(
+        self, request: HttpRequest, obj: Any | None = None, **kwargs: Any
+    ) -> type[BaseInlineFormSet]:
         kwargs = self._get_form_or_formset(request, obj, **kwargs)
         return super().get_formset(request, obj, **kwargs)
 
-    def get_fieldsets(self, request, obj=None):
+    def get_fieldsets(self, request: HttpRequest, obj: Any | None = None):
         # FIXME: If fieldsets are declared on an inline some kind of ghost
         # fieldset line with just the original model verbose_name of the model
         # is displayed above the new fieldsets.
@@ -397,13 +431,13 @@ class TabbedDjangoJqueryTranslationAdmin(TranslationAdmin):
 
     class Media:
         js = (
-            'admin/js/jquery.init.js',
-            'modeltranslation/js/force_jquery.js',
+            "admin/js/jquery.init.js",
+            "modeltranslation/js/force_jquery.js",
             mt_settings.JQUERY_UI_URL,
-            'modeltranslation/js/tabbed_translation_fields.js',
+            "modeltranslation/js/tabbed_translation_fields.js",
         )
         css = {
-            'all': ('modeltranslation/css/tabbed_translation_fields.css',),
+            "all": ("modeltranslation/css/tabbed_translation_fields.css",),
         }
 
 
@@ -417,10 +451,10 @@ class TabbedExternalJqueryTranslationAdmin(TranslationAdmin):
         js = (
             mt_settings.JQUERY_URL,
             mt_settings.JQUERY_UI_URL,
-            'modeltranslation/js/tabbed_translation_fields.js',
+            "modeltranslation/js/tabbed_translation_fields.js",
         )
         css = {
-            'screen': ('modeltranslation/css/tabbed_translation_fields.css',),
+            "screen": ("modeltranslation/css/tabbed_translation_fields.css",),
         }
 
 
