@@ -1,10 +1,13 @@
+from __future__ import annotations
+
 import copy
-from typing import Iterable
+from typing import Any, Sequence, cast
 
 from django import VERSION, forms
 from django.core.exceptions import ImproperlyConfigured
-from django.db.models import fields
+from django.db.models import Model, fields
 from django.utils.encoding import force_str
+from django.utils.functional import Promise
 from django.utils.translation import override
 
 from modeltranslation import settings as mt_settings
@@ -18,6 +21,8 @@ from modeltranslation.utils import (
 )
 from modeltranslation.widgets import ClearableWidgetWrapper
 
+from ._typing import Self
+
 SUPPORTED_FIELDS = (
     fields.CharField,
     # Above implies also CommaSeparatedIntegerField, EmailField, FilePathField, SlugField
@@ -56,7 +61,7 @@ class NONE:
     pass
 
 
-def create_translation_field(model, field_name, lang, empty_value):
+def create_translation_field(model: type[Model], field_name: str, lang: str, empty_value: Any):
     """
     Translation field factory. Returns a ``TranslationField`` based on a
     fieldname and a language.
@@ -69,22 +74,22 @@ def create_translation_field(model, field_name, lang, empty_value):
     If the class is neither a subclass of fields in ``SUPPORTED_FIELDS``, nor
     in ``CUSTOM_FIELDS`` an ``ImproperlyConfigured`` exception will be raised.
     """
-    if empty_value not in ('', 'both', None, NONE):
-        raise ImproperlyConfigured('%s is not a valid empty_value.' % empty_value)
-    field = model._meta.get_field(field_name)
+    if empty_value not in ("", "both", None, NONE):
+        raise ImproperlyConfigured("%s is not a valid empty_value." % empty_value)
+    field = cast(fields.Field, model._meta.get_field(field_name))
     cls_name = field.__class__.__name__
     if not (isinstance(field, SUPPORTED_FIELDS) or cls_name in mt_settings.CUSTOM_FIELDS):
-        raise ImproperlyConfigured('%s is not supported by modeltranslation.' % cls_name)
+        raise ImproperlyConfigured("%s is not supported by modeltranslation." % cls_name)
     translation_class = field_factory(field.__class__)
     return translation_class(translated_field=field, language=lang, empty_value=empty_value)
 
 
-def field_factory(baseclass):
-    class TranslationFieldSpecific(TranslationField, baseclass):
+def field_factory(baseclass: type[fields.Field]) -> type[TranslationField]:
+    class TranslationFieldSpecific(TranslationField, baseclass):  # type: ignore[valid-type, misc]
         pass
 
     # Reflect baseclass name of returned subclass
-    TranslationFieldSpecific.__name__ = 'Translation%s' % baseclass.__name__
+    TranslationFieldSpecific.__name__ = "Translation%s" % baseclass.__name__
 
     return TranslationFieldSpecific
 
@@ -108,7 +113,14 @@ class TranslationField:
     that needs to be specified when the field is created.
     """
 
-    def __init__(self, translated_field, language, empty_value, *args, **kwargs):
+    def __init__(
+        self,
+        translated_field: fields.Field,
+        language: str,
+        empty_value: Any,
+        *args: Any,
+        **kwargs: Any,
+    ) -> None:
         from modeltranslation.translator import translator
 
         # Update the dict of this field with the content of the original one
@@ -120,7 +132,7 @@ class TranslationField:
         self.language = language
         self.empty_value = empty_value
         if empty_value is NONE:
-            self.empty_value = None if translated_field.null else ''
+            self.empty_value = None if translated_field.null else ""
 
         # Default behaviour is that all translations are optional
         if not isinstance(self, fields.BooleanField):
@@ -133,7 +145,7 @@ class TranslationField:
         trans_opts = translator.get_options_for_model(self.model)
         if trans_opts.required_languages:
             required_languages = trans_opts.required_languages
-            if isinstance(trans_opts.required_languages, (tuple, list)):
+            if isinstance(required_languages, (tuple, list)):
                 # All fields
                 if self.language in required_languages:
                     # self.null = False
@@ -144,7 +156,7 @@ class TranslationField:
                 try:
                     req_fields = required_languages[self.language]
                 except KeyError:
-                    req_fields = required_languages.get('default', ())
+                    req_fields = required_languages.get("default", ())
                 if self.name in req_fields:
                     # TODO: We might have to handle the whole thing through the
                     # FieldsAggregationMetaClass, as fields can be inherited.
@@ -204,7 +216,7 @@ class TranslationField:
             )
             self.remote_field.field = self
 
-            if hasattr(self.remote_field.model._meta, '_related_objects_cache'):
+            if hasattr(self.remote_field.model._meta, "_related_objects_cache"):
                 del self.remote_field.model._meta._related_objects_cache
 
         # ForeignKey support - rewrite related_name
@@ -222,7 +234,7 @@ class TranslationField:
                 self.related_query_name = lambda: loc_related_query_name
             self.rel.related_name = build_localized_fieldname(current, self.language)
             self.rel.field = self
-            if hasattr(self.rel.to._meta, '_related_objects_cache'):
+            if hasattr(self.rel.to._meta, "_related_objects_cache"):
                 del self.rel.to._meta._related_objects_cache
         elif NEW_RELATED_API and self.remote_field and not self.remote_field.is_hidden():
             current = self.remote_field.get_accessor_name()
@@ -237,7 +249,7 @@ class TranslationField:
                 self.related_query_name = lambda: loc_related_query_name
             self.remote_field.related_name = build_localized_fieldname(current, self.language)
             self.remote_field.field = self
-            if hasattr(self.remote_field.model._meta, '_related_objects_cache'):
+            if hasattr(self.remote_field.model._meta, "_related_objects_cache"):
                 del self.remote_field.model._meta._related_objects_cache
 
     # Django 1.5 changed definition of __hash__ for fields to be fine with hash requirements.
@@ -245,20 +257,20 @@ class TranslationField:
     # original field and fields didn't get added to sets.
     # So here we override __eq__ and __hash__ to fix the issue while retaining fine with
     # http://docs.python.org/2.7/reference/datamodel.html#object.__hash__
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
         if isinstance(other, fields.Field):
             return self.creation_counter == other.creation_counter and self.language == getattr(
-                other, 'language', None
+                other, "language", None
             )
         return super().__eq__(other)
 
-    def __ne__(self, other):
+    def __ne__(self, other: object) -> bool:
         return not self.__eq__(other)
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         return hash((self.creation_counter, self.language))
 
-    def get_default(self):
+    def get_default(self) -> Any:
         with override(self.language):
             default = super().get_default()
             # we must *force evaluation* at this point, otherwise the lazy translatable
@@ -267,9 +279,11 @@ class TranslationField:
             #
             # force_str passes protected types as-is, which includes None, int, float,
             # datetime...
-            return force_str(default, strings_only=True)
+            if isinstance(default, Promise):
+                default = force_str(default, strings_only=True)
+            return default
 
-    def formfield(self, *args, **kwargs):
+    def formfield(self, *args: Any, **kwargs: Any) -> forms.Field:
         """
         Returns proper formfield, according to empty_values setting
         (only for ``forms.CharField`` subclasses).
@@ -298,23 +312,23 @@ class TranslationField:
                 from modeltranslation.forms import NullCharField
 
                 form_class = formfield.__class__
-                kwargs['form_class'] = type(
-                    'Null%s' % form_class.__name__, (NullCharField, form_class), {}
+                kwargs["form_class"] = type(
+                    "Null%s" % form_class.__name__, (NullCharField, form_class), {}
                 )
                 formfield = super().formfield(*args, **kwargs)
-            elif self.empty_value == 'both':
+            elif self.empty_value == "both":
                 from modeltranslation.forms import NullableField
 
                 form_class = formfield.__class__
-                kwargs['form_class'] = type(
-                    'Nullable%s' % form_class.__name__, (NullableField, form_class), {}
+                kwargs["form_class"] = type(
+                    "Nullable%s" % form_class.__name__, (NullableField, form_class), {}
                 )
                 formfield = super().formfield(*args, **kwargs)
                 if isinstance(formfield.widget, (forms.TextInput, forms.Textarea)):
                     formfield.widget = ClearableWidgetWrapper(formfield.widget)
         return formfield
 
-    def save_form_data(self, instance, data, check=True):
+    def save_form_data(self, instance: Model, data: Any, check: bool = True) -> None:
         # Allow 3rd-party apps forms to be saved using only translated field name.
         # When translated field (e.g. 'name') is specified and translation field (e.g. 'name_en')
         # not, we assume that form was saved without knowledge of modeltranslation and we make
@@ -325,21 +339,21 @@ class TranslationField:
         # Questionable fields are stored in special variable, which is later handled by clean_fields
         # method on the model.
         if check and self.language == get_language() and getattr(instance, self.name) and not data:
-            if not hasattr(instance, '_mt_form_pending_clear'):
+            if not hasattr(instance, "_mt_form_pending_clear"):
                 instance._mt_form_pending_clear = {}
             instance._mt_form_pending_clear[self.name] = data
         else:
             super().save_form_data(instance, data)
 
-    def deconstruct(self):
+    def deconstruct(self) -> tuple[str, str, Sequence[Any], dict[str, Any]]:
         name, path, args, kwargs = self.translated_field.deconstruct()
         if self.null is True:
-            kwargs.update({'null': True})
-        if 'db_column' in kwargs:
-            kwargs['db_column'] = self.db_column
+            kwargs.update({"null": True})
+        if "db_column" in kwargs:
+            kwargs["db_column"] = self.db_column
         return self.name, path, args, kwargs
 
-    def clone(self):
+    def clone(self) -> Self:
         from django.utils.module_loading import import_string
 
         name, path, args, kwargs = self.deconstruct()
@@ -353,8 +367,12 @@ class TranslationFieldDescriptor:
     """
 
     def __init__(
-        self, field, fallback_languages=None, fallback_value=NONE, fallback_undefined=NONE
-    ):
+        self,
+        field: fields.Field,
+        fallback_languages: dict[str, tuple[str, ...]] | None = None,
+        fallback_value: Any = NONE,
+        fallback_undefined: Any = NONE,
+    ) -> None:
         """
         Stores fallback options and the original field, so we know it's name
         and default.
@@ -372,7 +390,7 @@ class TranslationFieldDescriptor:
         instance.__dict__[self.field.name] = value
         if isinstance(self.field, fields.related.ForeignKey):
             instance.__dict__[self.field.get_attname()] = None if value is None else value.pk
-        if getattr(instance, '_mt_init', False) or getattr(instance, '_mt_disable', False):
+        if getattr(instance, "_mt_init", False) or getattr(instance, "_mt_disable", False):
             # When assignment takes place in model instance constructor, don't set value.
             # This is essential for only/defer to work, but I think it's sensible anyway.
             # Setting the localized field may also be disabled by setting _mt_disable.
@@ -431,7 +449,9 @@ class TranslatedRelationIdDescriptor:
     ForeignKey field.
     """
 
-    def __init__(self, field_name: str, fallback_languages: Iterable[str]):
+    def __init__(
+        self, field_name: str, fallback_languages: dict[str, tuple[str, ...]] | None
+    ) -> None:
         self.field_name = field_name  # The name of the original field (excluding '_id')
         self.fallback_languages = fallback_languages
 
@@ -463,7 +483,9 @@ class TranslatedManyToManyDescriptor:
     A descriptor used to return correct related manager without language fallbacks.
     """
 
-    def __init__(self, field_name, fallback_languages):
+    def __init__(
+        self, field_name: str, fallback_languages: dict[str, tuple[str, ...]] | None
+    ) -> None:
         self.field_name = field_name  # The name of the original field
         self.fallback_languages = fallback_languages
 
@@ -487,16 +509,16 @@ class LanguageCacheSingleObjectDescriptor:
     accessor = None  # needs to be set on instance
 
     @property
-    def cache_name(self):
+    def cache_name(self) -> str:
         """
         Used in django 1.x
         """
         lang = get_language()
-        cache = build_localized_fieldname(self.accessor, lang)
+        cache = build_localized_fieldname(self.accessor, lang)  # type: ignore[arg-type]
         return "_%s_cache" % cache
 
-    def get_cache_name(self):
+    def get_cache_name(self) -> str:
         """
         Used in django > 2.x
         """
-        return build_localized_fieldname(self.accessor, get_language())
+        return build_localized_fieldname(self.accessor, get_language())  # type: ignore[arg-type]
