@@ -1,43 +1,43 @@
-# -*- coding: utf-8 -*-
-from django.db.models import F, Q, ManyToManyField
-from django.core.management.base import BaseCommand, CommandError
+from typing import Any
+
+from django.core.management.base import BaseCommand, CommandParser, CommandError
+from django.db.models import F, ManyToManyField, Q
 
 from modeltranslation.settings import AVAILABLE_LANGUAGES, DEFAULT_LANGUAGE
 from modeltranslation.translator import translator
 from modeltranslation.utils import build_localized_fieldname
 
-
 COMMASPACE = ", "
 
 
 class Command(BaseCommand):
     help = (
-        'Updates empty values of translation fields using'
-        ' values from original fields (in all translated models).'
+        "Updates empty values of translation fields using"
+        " values from original fields (in all translated models)."
     )
 
-    def add_arguments(self, parser):
+    def add_arguments(self, parser: CommandParser) -> None:
         parser.add_argument(
-            'app_label',
-            nargs='?',
-            help='App label of an application to update empty values.',
+            "app_label",
+            nargs="?",
+            help="App label of an application to update empty values.",
         )
         parser.add_argument(
-            'model_name',
-            nargs='?',
-            help='Model name to update empty values of only this model.',
+            "model_name",
+            nargs="?",
+            help="Model name to update empty values of only this model.",
         )
         parser.add_argument(
-            '--language',
-            action='store',
+            "--language",
+            action="store",
             help=(
-                'Language translation field the be updated.'
-                ' Default language field if not provided'
+                "Language translation field the be updated."
+                " Default language field if not provided"
             ),
         )
 
-    def handle(self, *args, **options):
-        verbosity = options['verbosity']
+    def handle(self, *args: Any, **options: Any) -> None:
+        verbosity: int = options["verbosity"]
         if verbosity > 0:
             self.stdout.write("Using default language: %s" % DEFAULT_LANGUAGE)
 
@@ -46,30 +46,30 @@ class Command(BaseCommand):
         models = [m for m in models if not m._meta.proxy and m._meta.managed]
 
         # optionally filter by given app_label
-        app_label = options['app_label']
+        app_label = options["app_label"]
         if app_label:
             models = [m for m in models if m._meta.app_label == app_label]
 
         # optionally filter by given model_name
-        model_name = options['model_name']
+        model_name = options["model_name"]
         if model_name:
             model_name = model_name.lower()
             models = [m for m in models if m._meta.model_name == model_name]
 
         # optionally defining the translation field language
-        lang = options.get('language') or DEFAULT_LANGUAGE
+        lang = options.get("language") or DEFAULT_LANGUAGE
         if lang not in AVAILABLE_LANGUAGES:
             raise CommandError(
                 "Cannot find language '%s'. Options are %s."
                 % (lang, COMMASPACE.join(AVAILABLE_LANGUAGES))
             )
         else:
-            lang = lang.replace('-', '_')
+            lang = lang.replace("-", "_")
 
         if verbosity > 0:
             self.stdout.write(
                 "Working on models: %s"
-                % ', '.join(
+                % ", ".join(
                     ["{app_label}.{object_name}".format(**m._meta.__dict__) for m in models]
                 )
             )
@@ -83,7 +83,7 @@ class Command(BaseCommand):
                 def_lang_fieldname = build_localized_fieldname(field_name, lang)
 
                 # We'll only update fields which do not have an existing value
-                q = Q(**{def_lang_fieldname: None})
+                q = Q(**{f"{def_lang_fieldname}__isnull": True})
                 field = model._meta.get_field(field_name)
                 if isinstance(field, ManyToManyField):
                     trans_field = getattr(model, def_lang_fieldname)
@@ -96,9 +96,9 @@ class Command(BaseCommand):
                             for inst in getattr(model, field_name).through.objects.all()
                         )
                     continue
-                if field.empty_strings_allowed:
+                if field.empty_strings_allowed:  # type: ignore[union-attr]
                     q |= Q(**{def_lang_fieldname: ""})
 
-                model._default_manager.filter(q).rewrite(False).order_by().update(
+                model._default_manager.filter(q).rewrite(False).order_by().update(  # type: ignore[attr-defined]
                     **{def_lang_fieldname: F(field_name)}
                 )
